require 'yaml'
require 'puppet/indirector/node/yaml_device'
require 'timeout'
require 'json'
require 'fileutils'
require 'open3'
require 'io/wait'
require 'puppet/face'
require 'pathname'

module Puppet
  module ASM
    module Util

      # processes the provided node by doing the following:
      #   - populate ENC data
      #   - run puppet
      #   - check results
      def process_a_node(certname, type, resource_file, always_override, remove_node_file=true)

        unless File.exists?(resource_file)
          raise(Exception, "Resource file #{resource_file} does not exist")
        end

        resources = YAML.load_file(resource_file)

        num_resources = resources.size

        node_file    = Puppet::Node::YamlDevice.device_file(certname)

        # set up a staging directory where we can stage the creation
        # of a node yaml file and compile a catalog before we actually
        # override the real node yaml
        staging_file =  staging_file_name(node_file, certname)
        staging_certname = "staging_#{certname}"

        if File.exists?(node_file) and ! always_override
          append_resources(node_file, certname, resources, staging_file, "staging_#{certname}")
        else
          add_resources(staging_file, "staging_#{certname}", resources)
        end

        # Do not attempt to compile for puppet apply, this node may not have puppet master
        if type == 'device' && !pre_validate_resource_compile(staging_certname, node_file, resources)
          raise(Exception, "Provided resources did not result in a catalog that could be compiled")
        end

        # now do it for realz
        if File.exists?(node_file) and ! always_override
          append_resources(node_file, certname, resources)
        else
          add_resources(node_file, certname, resources)
        end

        state_dir = state_dir(certname)

        common_options = "--certname #{certname} --statedir #{state_dir} --no-usecacheonfailure"
        if Puppet::Util::Log.level == 'debug'
          common_options = "#{common_options} --debug"
        end
        if Puppet[:trace]
          common_options = "#{common_options} --debug --trace"
        end
        if Puppet[:noop]
          common_options = "#{common_options} --noop"
        end

        unless File.exists?(state_dir)
          Puppet.info("Creating state directory #{state_dir}")
          FileUtils.mkdir_p(state_dir)
        end

        if type == 'device'
          cmd = "timeout 3600 puppet device #{common_options} --deviceconfig #{Puppet[:confdir]}/devices/#{certname}.conf --no-daemonize --verbose"
        elsif type == 'apply'
          cmd = "timeout 3600 puppet apply #{common_options} -e ''"
        elsif type == 'agent'
          # do not run any command for agent mode
          # for now, just populate the data
          cmd = nil
        else
          raise(Exception, "Unexpected puppet run type: #{type}")
        end
        results = {}
        if cmd
          run_command(cmd)
          results = process_results(certname)
          num_failures = results.delete(:num_failures)

          errors = nil
          if !always_override
            errors = filter_errors(certname, resources, state_dir, num_failures)
          else
            errors = {
              :run_failures   => num_failures,
              :other_failures => 0
            }
          end
          results.merge!(errors)

          puts "Results: For #{num_resources} resources." +
             " #{results[:run_failures]} from our run failed." +
             " #{results[:other_failures]} not from our run failed." +
             " #{results[:num_updates]} updated successfully."

          if type == 'device'
            move_last_run_report(certname)
          end

          change_report_permissions(certname)
          raise(Exception, "#{results[:run_failures]} failed") if results[:run_failures] != 0
        end
        results
      ensure
        if remove_node_file && node_file && File.exist?(node_file)
          File.unlink(node_file)
        end
      end

      # figure out for a run:
      # * did the run fail
      # * did any resources from the run fail
      # * how many resources did we submit and how many were updated
      #
      def process_results(certname)
        last_run_summary = last_run_summary_file(certname)
        unless File.exists?(last_run_summary)
          raise(Exception, "Expected file #{last_run_summary} does not exist")
        end
        events = YAML.load_file(last_run_summary)['events']
        {:num_failures => events['failure'],
         :num_updates  => events['success']}
      end

      def change_report_permissions(certname)
        last_run_report = last_run_report_file(certname)
        if File.exists?(last_run_report)
          File.chmod(0644, last_run_report)
        end
      end

      def move_last_run_report(certname)
        report_file = latest_report_file(certname)
        file_dest = last_run_report_file(certname)
        FileUtils.cp(report_file,file_dest)
      end

      # get the location of a Puppet runs state directory
      def state_dir(certname)
        File.join(Puppet[:statedir], 'state', certname)
      end

      # get the location of the last run summary file
      # this method primarily exists to make stubbing easier for testing
      def last_run_summary_file(certname)
        File.join(state_dir(certname), 'last_run_summary.yaml')
      end

      # get the location of the last run report
      def last_run_report_file(certname)
        File.join(state_dir(certname), 'last_run_report.yaml')
      end

      # create a new file and add resources to it
      # override any existing files
      def add_resources(node_file, certname, resources)
        unless File.exists?(File.dirname(node_file))
          raise(Exception, "Directory #{File.dirname(node_file)} does not exist")
        end
        File.open(node_file, 'w') do |fh|
          Timeout::timeout(1) { fh.flock(File::LOCK_EX) }
          fh.puts({certname => {'resources' => resources}}.to_yaml)
          fh.flock(File::LOCK_UN)
        end
      end

      # append resources to an existing device description
      # fail if a resource is already defined
      def append_resources(node_file, certname, new_resources, new_file=nil, new_certname=nil)
        new_certname ||= certname
        new_file     ||= node_file
        if File.exists?(node_file) and old = YAML.load_file(node_file)
          unless old_resources = (old[certname] || {})['resources']
            raise("The loaded yaml does not seem to have the certname: #{certname} correctly entered")
          end

          new_resources.each do |type, resources|
            if old_type = old_resources[type]
              resources.each do |title, attrs|
                if old_type[title]
                  raise(Exception, "Resource #{type}/#{title} already defined." +
                                   " You have a conflict between service templates")
                end
                old_resources[type][title] = attrs
              end
            else
              old_resources[type] = resources
            end
          end
          File.open(new_file, 'w') do |fh|
            fh.write({new_certname => {'resources' => old_resources}}.to_yaml)
          end
        else
          File.open(new_file, 'w') do |fh|
            fh.write({new_certname => {'resources' => new_resources}}.to_yaml)
          end
        end

      end

      def run_command(cmd)
        Puppet.info("Running command: #{cmd}")
        Open3.popen2e(cmd) do |stdin, out, wait_thr|
          stdin.close

          puts out.gets until out.eof?

          raise(Exception, "#{cmd} failed") unless wait_thr.value.exitstatus == 0
        end
      end

      #
      # make sure that you can compile a catalog for a specified certname
      #
      def pre_validate_resource_compile(certname, node_file, resources)
        require 'asm/util'
        cmd = "puppet catalog find #{certname}"
        Puppet.debug("Verifying catalog with command:: #{cmd}")
        results = ::ASM::Util.run_command_simple(cmd)
        unless results['exit_status'] == 0
          raise(Exception, "Puppet catalog compile failed: #{results['stderr']}}|#{results['stdout']}")
        end
        true
      end

      def staging_file_name(node_file, certname)
        stage_dir = File.dirname(node_file)
        basename  = File.basename(node_file, '.yaml')
        unless basename == certname
          raise(Exception, "Basename #{basename} from node data does not match certname #{certname}}")
        end
        File.join(stage_dir, "staging_#{basename}.yaml")
      end

      def latest_report_file(certname)
        report_dir = Puppet[:reportdir]
        latest_report = Dir[File.join(report_dir, certname, '*.yaml' )].sort.last
      end

      #
      # figure out how many errors are from our run and how many are not
      # from our run
      #
      def filter_errors(certname, resources, state_dir, num_failures)
        errors = {
          :run_failures   => 0,
          :other_failures => 0
        }
        if num_failures > 0
          latest_report = latest_report_file(certname)
          report = YAML.load_file(latest_report)
          # iterate through each of the resources
          report.resource_statuses.each do |name, status|
            if status.failed
              if name =~ /^(\S+?)\[(\S+)\]$/
                Puppet.info("Resource #{name} has failed")
                type  = $1.downcase
                title = $2.downcase
                if (resources[type] || {})[title]
                  Puppet.warning("Failed resource #{name} found that we are managing")
                  errors[:run_failures] += 1
                else
                  Puppet.warning("Failed resource #{name} found that we are not managing")
                  errors[:other_failures] += 1
                end
              else
                raise(Exception, "Unexpected resource #{name}")
              end
            end
          end
        end
        unless num_failures == errors[:run_failures] + errors[:other_failures]
          raise(Exception, "Somehow got a report with a different number of faillures")
        end
        errors
      end

    end
  end
end
