require File.join(
  File.dirname(__FILE__), '..', '..',
  'puppet/asm/util.rb'
)
include Puppet::ASM::Util
Puppet::Face.define(:asm, '0.0.1') do
  action :process_node do

    summary 'Process the parts of a service profile that are related to a node'

    description <<-EOT
      This command will process new device entries.
      It expects resources for the specified certname to be
      in the provided file in the form:
        type:
          title1:
            attr1: value1
          tilte2
            attr1: value1
        type1:
           ...
      These resources are written to the node's specific
      yaml file, then either puppet device or apply is run and
      the results are checked.
    EOT

    arguments 'certname'

    option '--filename FILENAME' do
      summary 'File where node resources are defined'
    end

    option '--run_type RUN_TYPE' do
      summary 'The type of Puppet run to perform (accepts apply or device)'
      default_to { 'device' }
    end

    option '--always-override' do
      summary 'Always override config files even if they exist'
      default_to { false }
    end

    when_invoked do |name, options|
      unless options[:filename]
        raise(Exception, 'resources file must be provided with --filename')
      end
      process_a_node(name, options[:run_type], options[:filename], options[:always_override])
    end

  end

  action :reinventory_facts do

    description <<-EOT

This command can be used to collect facts from a network device and
post them to the puppet master.

It accepts certname as an argument.

It assumes that the device.conf file is location at:

    /etc/puppetlabs/puppet/devices/certname.conf

It assumes that the certificates for the device are stored at:

   /var/opt/lib/pe-puppet/devices/certname/ssl

The following configuration must be supplied to auth.conf to get the facts upload working

    path ~ ^/facts/([^/]+)$
    method save
    allow $1

    EOT

    arguments 'certname'

    when_invoked do |certname, options|
      require 'puppet/asm/device'
      Puppet::ASM::Device.update_inventory(certname)
    end

  end

  action :summarize_report do

    option '--infile INFILE' do
      summary 'Path to logfile to be summarized'
    end

    option '--outfile OUTFILE' do
      summary 'File where summarized log should be written'
    end

    when_invoked do |options|
      raise(Exception, 'infile option must be specified') unless options[:infile]
      raise(Exception, 'Outfile option must be specified') unless options[:outfile]
      report = YAML.load_file(options[:infile])
      ignore_types = ['schedule', 'filebucket']
      result_hash  = {}
      report.resource_statuses.each do |name, status|
        if name =~ /^(\S+?)\[(\S+)\]$/
          type  = $1.downcase
          title = $2.downcase
          unless ignore_types.include?(type)
            if status.failed?
              result_hash[name] = 'FAILED'
            else
              if status.out_of_sync?
                if status.changed?
                  result_hash[name] = 'UPDATED'
                elsif status.restarted?
                  result_hash[name] = 'RESTARTED'
                else
                  results_hash[name] = 'UNEXPECTED'
                end
              else
                result_hash[name] = 'NO_UPDATE'
              end
            end
          end
        else
          raise(Exception, "Unexpected resource #{name}")
        end
      end
      File.open(options[:outfile], 'w') do |fh|
        fh.puts(result_hash.to_yaml)
      end
    end
  end
end
