require 'puppet/indirector/yaml'
require 'puppet/node'
require 'yaml'

class Puppet::Node::YamlDevice < Puppet::Indirector::Yaml
  desc <<-EOT

    This ENC can be used to specify what resources should be
    configured for a specific node. It can also specify parameters
    and classes like a typical ENC.

    It pulls this classification information from the file:

      $confdir/node_data/<certname>.yaml

    The default location of $confdir is /etc/puppet/ for root
    or ~/.puppet for a system user.

    The data for this file is of the form:

      certname1:
        resources:
          user:
            dan:
              ensure: present
      certname2:
        resources:
          user:
            bob:
              ensure: present


  EOT

  def find(request)
    node_name = request.key
    node = Puppet::Node.new(request.key)
    yaml_file = self.class.device_file(node.name)
    if File.exists?(yaml_file)
      device_hash = YAML.load_file(yaml_file)
      unless device_hash
        raise(Puppet::Error, "Yaml file #{yaml_file} was empty")
      end
      if device_info = device_hash[node_name]
        # TODO for now, I will not allow params or classes to be set here
        # but ,we will probably need it later
        node.parameters = (device_info['parameters'] || {})
        node.classes    = (device_info['classes'] || {})
        if resources = device_info['resources']
          if node.classes
            unless node.classes.class == Hash
              #
              # we have to pass in classes as a Hash b/c we have to set the resource
              # parameter in our class. I will fail if classes were specified as an array
              # b/c otherwise, I have to convert it into a hash which potentially effects
              # things if the user was expecting an array
              #
              raise(Puppet::Error, 'Cannot add resources if an array of classes was passed')
            end
          end
          node.classes.merge!(
            'asm::resource_wrapper' => {'resources' => resources}
          )
        end
      else
        Puppet.warning("Did not find node: #{node_name} in #{yaml_file}")
      end
    else
      Puppet.warning("Yaml file #{yaml_file} does not exist")
    end
    node.fact_merge
    node
  end

  def self.device_file(certname)
    File.expand_path(File.join(Puppet[:confdir], 'node_data', "#{certname}.yaml"))
  end

end
