# This defined resource creates vCenter virtual machines
#
# Parameters:
# - $cpu_count - Number of CPU's assigned to the new Virtual Machine.
# - $memory_in_mb - Amount of memory to be assigned to provisioned VM.
# - $disksize_in_kb - Capacity of the virtual disk (in KB). Defaults to 32GB
# - $vnic_count - Nic Count that needs to be added in the new Virtual Machine.
# - $cluster - vCenter cluster the VM should be created in.
# - $os_type - Name of Guest OS type of Clone VM. Should be windows or linux.
# - $guest_os_id - Guest operating system identifier. User can get the
#                  guestid from following url:
#                  http://pubs.vmware.com/vsphere-55/index.jsp?topic=%2Fcom.vmware.wssdk.apiref.doc%2Fvim.vm.GuestOsDescriptor.GuestOsIdentifier.html
# - $datacenter - vCenter datacenter the VM should be created in.
# - $datastore - vCenter datastore the VM should use.
# - $source - VM or template name to clone from
# - $source_datacenter - datacenter the VM or template to clone from is in
# - $clone_type - either vm or template depending on what the source is
# - $vcenter_id - Id that can be used to locate device.conf file
# - $vcenter_options - Connection options such as { 'insecure' => 'true' }.
# - $ensure - Whether the vm should be present or absent.
#
define asm::vm::vcenter(
  $ensure,
  $cpu_count,
  $memory_in_mb,
  $cluster,
  $os_type              = undef,
  $os_guest_id          = undef,
  $datacenter,
  $vcenter_id,
  $vcenter_options,
  $clone_type           = undef,
  $source               = undef,
  $source_datacenter    = undef,
  $datastore            = undef,
  $vm_custom_spec       = undef,
  $scsi_controller_type = 'VMware Paravirtual',
  $disksize_in_gb       = 32,
  $network_interfaces   = [{portgroup => 'VM Network' , nic_type => 'vmxnet3'}],
  $skip_local_datastore = false,
  $default_gateway      = undef
) {
  transport { 'vcenter':
    name     => $vcenter_id,
    options  => $vcenter_options,
    provider => 'device_file',
  }

  vc_vm { $name:
    ensure               => $ensure,
    datacenter           => $datacenter,
    template             => $source,
    template_datacenter  => $source_datacenter,
    memory_mb            => $memory_in_mb,
    num_cpus             => $cpu_count,
    cluster              => $cluster,
    datastore            => $datastore,
    disk_size            => $disksize_in_gb * 1024 * 1024,
    skip_local_datastore => $skip_local_datastore,
    disk_format          => 'thin',
    scsi_controller_type => $scsi_controller_type,
    guest_customization  => false,
    guest_custom_spec    => $vm_custom_spec,
    guest_type           => $os_type,
    guestid              => $os_guest_id,
    power_state          => 'poweredOn',
    network_interfaces   => $network_interfaces,
    transport            => Transport['vcenter'],
    inclusive            => true
  }
}
