#!/bin/env ruby

# Simple script to transform a node data yaml file into a puppet
# manifest file which can then be executed directly with "puppet
# apply". Useful for debugging deployment resource files.

require 'yaml'

def escape_string(string)
  # Simple escaping, replace \ and ' with \\ and \'
  # UPDATE: back-slashes don't seem to need to be escaped
  "'" + string.gsub(/(['])/, '\\\\\1') + "'"
end

def escape(val)
  if val.is_a?(String)
    escape_string(val)
  elsif val.is_a?(Integer)
    escape_string(val.to_s)
  elsif !!val == val # boolean
    escape_string(val.to_s)
  elsif val.is_a?(Hash)
    '{ ' + val.map { |k, v| "#{escape_string(k.downcase)} => #{escape(v)}" }.join(",\n        ") + ' }'
  elsif val.is_a?(Array)
    "[ " + val.map { |x| escape(x) }.join(', ') + " ]"
  elsif val.nil?
    "''"
  else
    raise(Exception, "Unknown value for manifest: #{val}")
  end
end

def write_manifest(resource_data, fh, node_name = nil)
  node_name = escape_string(node_name) unless node_name == 'default'
  if node_name
    fh.puts("node #{node_name} {\n\n")
  end
  resource_data.each do |type, resources|
    resources.each do |title, params|
      fh.puts("#{type} { #{escape(title)}:")
      params.each do |key, val|
        fh.puts("  #{key} => #{escape(val)},")
      end
      fh.puts('}')
      fh.puts('')
    end
  end
  if node_name
    fh.puts("\n}\n")
  end
end

unless ARGV.length == 1 || ARGV.length == 2
  $stderr.puts <<EOT
USAGE: yaml2manifest.rb [yaml config file] (node name)

Converts yaml node classification data into a puppet manifest
that can be run with puppet apply.

If (node name) is passed the manifest will be wrapped in
"node [NAME] { }" so that it can be saved directly to
/etc/puppetlabs/puppet/manifests/site.pp If that is done and
there is not /etc/puppetlabs/puppet/node_data/[CERTNAME].yaml
file, then the resulting manifest can be executed with puppet
apply with this command:

    puppet apply -e --certname [CERTNAME]

EOT
  exit(false)
end

file = ARGV[0]
unless File.exists?(file)
  $stderr.puts("File does not exist: #{file}")
  exit(false)
end

node_data = YAML.load_file(file)

write_manifest(node_data, $stdout, ARGV[1])
