require 'spec_helper'
require 'puppet/asm/util'
require 'asm/util'
require 'tempfile'

describe 'yaml device node indirection' do

  before do
    Puppet.stubs(:[]).with(:confdir).returns(Dir.tmpdir)
    Puppet.stubs(:[]).with(:statedir).returns(Dir.tmpdir)
    Puppet.stubs(:[]).with(:trace)
    Puppet.stubs(:[]).with(:noop)
  end

  include Puppet::ASM::Util

  def fixtures_dir
    File.join(File.dirname(__FILE__), '..', '..', '..', 'fixtures')
  end

  def tmp_file(contents, id='foo')
    good_file = Tempfile.new(id)
    filename  = good_file.path
    good_file.write(contents)
    good_file.close
    filename
  end

  describe 'when creating a node file' do

    it 'should add the resources correctly to the file' do
      tmp_file = Tempfile.new('asm_util_test')
      path     = tmp_file.path
      tmp_file.unlink
      add_resources(path, 'node_name', {
        'user' => {'dan' => {'ensure' => 'present'}}
      })
      written_resources = YAML.load_file(path)
      written_resources['node_name']['resources']['user']['dan']['ensure'].should == 'present'
    end

    it 'should override existing files' do
      existing_file = tmp_file(<<-EOT
node_name:
  resources:
    group:
      adm:
        ensure: present
EOT
      )
      add_resources(existing_file, 'node_name', {
        'user' => {'dan' => {'ensure' => 'present'}}
      })
      written_resources = YAML.load_file(existing_file)
      written_resources['node_name']['resources']['user']['dan']['ensure'].should == 'present'
      written_resources['node_name']['resources']['group'].should be_nil
    end

  end

  describe 'when appending to a resource' do

    it 'should create a new file when current does not exist' do
      tmp_file = Tempfile.new('asm_util_test')
      path     = tmp_file.path
      tmp_file.unlink
      append_resources(path, 'node_name',
        {'user' => {'dan' => {'ensure' => 'present'}}}
      )
      written_resources = YAML.load_file(path)
      written_resources['node_name']['resources']['user']['dan']['ensure'].should == 'present'
    end

    it 'should add resource contents when current file is empty' do
      existing_file = tmp_file('')
      append_resources(existing_file, 'node_name',
        {'user' => {'dan' => {'ensure' => 'present'}}}
      )
      written_resources = YAML.load_file(existing_file)
      written_resources['node_name']['resources']['user']['dan']['ensure'].should == 'present'
    end

    it 'should append new types' do
      existing_file  = tmp_file(<<-EOT
node_name:
  resources:
    group:
      adm:
        ensure: present
EOT
      )
      append_resources(existing_file, 'node_name',
        {'user' => {'dan' => {'ensure' => 'present'}}}
      )
      written_resources = YAML.load_file(existing_file)['node_name']['resources']
      written_resources['user']['dan']['ensure'].should == 'present'
      written_resources['group']['adm']['ensure'].should == 'present'

    end

    it 'it should append new resources' do
      existing_file  = tmp_file(<<-EOT
node_name:
  resources:
    user:
      adm:
        ensure: present
EOT
      )
      append_resources(existing_file, 'node_name',
        {'user' => {'dan' => {'ensure' => 'present'}}}
      )
      written_resources = YAML.load_file(existing_file)['node_name']['resources']
      written_resources['user']['dan']['ensure'].should == 'present'
      written_resources['user']['adm']['ensure'].should == 'present'

    end

    it 'should fail if the resource already exists in the file' do
      expect do
        existing_file  = tmp_file(<<-EOT
node_name:
  resources:
    user:
      dan:
        ensure: present
EOT
      )
        append_resources(existing_file, 'node_name',
          {'user' => {'dan' => {'ensure' => 'present'}}}
        )
      end.to raise_error(Exception, /You have a conflict between service templates/)
    end

  end

  describe 'when processing results' do

    it 'should fail if the last run summary file does not exist' do
      no_file = Tempfile.new('no_file')
      path    = no_file.path
      no_file.close
      no_file.unlink
      self.expects(:last_run_summary_file).with('node_name').returns(path)
      expect do
        self.process_results('node_name')
      end.to raise_error(Exception, /does not exist/)
    end

    it 'should parse results' do
      results_file  = tmp_file(<<-EOT
  events:
    failure: 2
    success: 4
    total: 5
EOT
      )
      self.expects(:last_run_summary_file).with('node_name').returns(results_file)
      results = process_results('node_name')
      results[:num_failures].should == 2
      results[:num_updates].should == 4
    end

  end

  describe 'when processing nodes' do

    it 'should fail if resource file does not exist' do
      no_file = Tempfile.new('no_file')
      path    = no_file.path
      no_file.close
      no_file.unlink
      expect do
        process_a_node('node_name', 'device', path, false)
      end.to raise_error(Exception, /does not exist/)
    end

    describe 'when it does not immediately fail' do

      before do
        tmp_dir = Dir.tmpdir
        existing_node_file = File.join(tmp_dir, 'node_name.yaml')
        File.open(existing_node_file, 'w') do |fh|
          fh.puts(<<-EOT
  node_name:
    resources:
      user:
        dan:
          ensure: present
  EOT
          )
        end
        Puppet::Node::YamlDevice.expects(:device_file).with('node_name').returns(existing_node_file)
        @resource_file = tmp_file(<<-EOT
  user:
    danny:
      ensure: present
  group:
    bob:
      ensure: absent
  EOT
        )
      end

      it 'should run puppet apply when type is apply' do
        cmd = "timeout 3600 puppet apply --certname node_name --statedir %s --no-usecacheonfailure -e ''" %
          File.join(Dir.tmpdir, "state", "node_name")
        self.expects(:run_command).with(cmd                  )
        self.expects(:process_results).with('node_name').returns(
          :num_failures => 0,
          :num_updates  => 1
        )
        process_a_node('node_name', 'apply', @resource_file, false)
      end

      it 'should not run a command when type is agent' do
        self.expects(:run_command).never
        process_a_node('node_name', 'agent', @resource_file, false)
      end

      it 'should run puppet device when the type is device' do
        cmd = "timeout 3600 puppet device --certname node_name --statedir %s --no-usecacheonfailure --deviceconfig %s --no-daemonize --verbose" %
          [File.join(Dir.tmpdir, "state", "node_name"), File.join(Dir.tmpdir, "devices", "node_name.conf")]
        self.expects(:run_command).with(cmd)
        ASM::Util.expects(:run_command_simple).with('puppet catalog find staging_node_name').returns({'exit_status' => 0})
        self.expects(:process_results).with('node_name').returns(
          :num_failures  => 0,
          :num_updates   => 1
        )
        self.stubs(:latest_report_file).returns(File.join(fixtures_dir, 'node_name', '201401151818.yaml'))
        process_a_node('node_name', 'device', @resource_file, false)
      end

      it 'should fail for invalid types' do
        expect do
          process_a_node('node_name', 'foo', @resource_file, false)
        end.to raise_error(Exception, /Unexpected puppet run type/)
      end

      it 'should override existing files when always override is provided' do
        self.expects(:run_command).with(
          "timeout 3600 puppet apply --certname node_name --statedir %s --no-usecacheonfailure -e ''" % File.join(Dir.tmpdir, "state", "node_name")
        )
        self.expects(:process_results).with('node_name').returns(
          :num_failures => 0,
          :num_updates  => 1
        )
        self.stubs(:add_resources)
        process_a_node('node_name', 'apply', @resource_file, true)
      end

    end
  end

  describe 'when puppet errors arise' do

    before do
      @node_data_dir  = Dir.tmpdir
      @node_data_file = File.join(@node_data_dir, 'node_name.yaml')
      File.open(@node_data_file, 'w') do |fh|
       fh.write(<<-EOT
node_name:
  resources:
    group:
      adm:
        ensure: present
EOT
        )
      end
      FileUtils.mkdir_p(File.join(@node_data_dir, '2'))
      @node_data_file2 = File.join(@node_data_dir, '2', 'node_name.yaml')
      File.open(@node_data_file2, 'w') do |fh|
       fh.write(<<-EOT
node_name:
  resources:
    host:
      bob:
        ensure: present
EOT
        )
      end
      @last_run_summary = File.join(@node_data_dir, 'last_run_summary.yaml')
      File.open(@last_run_summary, 'w') do |fh|
        fh.write(<<-EOT
---
  version:
    config: 1389809931
    puppet: "3.3.3"
  resources:
    changed: 1
    failed: 1
    failed_to_restart: 0
    out_of_sync: 1
    restarted: 0
    scheduled: 0
    skipped: 0
    total: 9
  time:
    config_retrieval: 0.754562657
    filebucket: 0.000206985
    host: 0.022219227
    schedule: 0.001520846
    total: 0.7795236750000001
    user: 0.00101396
    last_run: 1389809934
  changes:
    total: 1
  events:
    failure: 1
    success: 1
    total: 2
EOT
        )
      end
      @last_run_summary2 = File.join(@node_data_dir, '2', 'last_run_summary.yaml')
      FileUtils.mkdir_p(File.join(@node_data_dir, '2'))
      File.open(@last_run_summary2, 'w') do |fh|
        fh.write(<<-EOT
---
  version:
    config: 1389809931
    puppet: "3.3.3"
  resources:
    changed: 1
    failed: 2
    failed_to_restart: 0
    out_of_sync: 1
    restarted: 0
    scheduled: 0
    skipped: 0
    total: 9
  time:
    config_retrieval: 0.754562657
    filebucket: 0.000206985
    host: 0.022219227
    schedule: 0.001520846
    total: 0.7795236750000001
    user: 0.00101396
    last_run: 1389809934
  changes:
    total: 1
  events:
    failure: 2
    success: 1
    total: 2
EOT
        )

      end
    end

    it 'should fail when the errors are ours' do
      self.stubs(:run_command).returns('something')
      self.stubs(:state_dir).returns(@node_data_dir)
      self.stubs(:latest_report_file).returns(File.join(fixtures_dir, 'node_name', '201401151818.yaml'))
      Puppet::Node::YamlDevice.expects(:device_file).with('node_name').returns(@node_data_file)
      resource_file = tmp_file(<<-EOT
host:
  bob:
    ensure: present
      EOT
      )
      expect do
        process_a_node('node_name', 'apply', resource_file, false)
      end.to raise_error(Exception, '1 failed')
    end

    it 'should fail and not append resouces if the result cannot be compiled' do
      ::ASM::Util.expects(:run_command_simple).with('puppet catalog find staging_node_name').returns({'exit_status' => 1})
      self.stubs(:run_command).returns('something')
      self.stubs(:state_dir).returns(@node_data_dir)
      self.stubs(:latest_report_file).returns(File.join(fixtures_dir, 'node_name', '201401151818.yaml'))
      Puppet::Node::YamlDevice.expects(:device_file).with('node_name').returns(@node_data_file)
      resource_file = tmp_file(<<-EOT
host:
  bob:
    ensure: present
      EOT
      )
      expect do
        process_a_node('node_name', 'device', resource_file, false, false)
      end.to raise_error(Exception, /Puppet catalog compile failed/)
      written_resources = YAML.load_file(@node_data_file)
      written_resources['node_name']['resources']['group']['adm']['ensure'].should == 'present'
      written_resources['node_name']['resources']['host'].should be_nil
    end

    it 'should only warn if the failures are not related to us' do
      self.stubs(:run_command).returns('something')
      self.stubs(:state_dir).returns(File.join(@node_data_dir))
      self.stubs(:latest_report_file).returns(File.join(fixtures_dir, 'node_name', '201401151818.yaml'))
      Puppet::Node::YamlDevice.expects(:device_file).with('node_name').returns(@node_data_file2)
      resource_file = tmp_file(<<-EOT
host:
  dan:
    ensure: present
      EOT
      )
      process_a_node('node_name', 'apply', resource_file, false)
    end

    it 'should fail when the failure numbers do not match up' do
      self.stubs(:run_command).returns('something')
      self.stubs(:state_dir).returns(File.join(@node_data_dir, '2'))
      self.stubs(:latest_report_file).returns(File.join(fixtures_dir, 'node_name', '201401151818.yaml'))
      Puppet::Node::YamlDevice.expects(:device_file).with('node_name').returns(@node_data_file)
      resource_file = tmp_file(<<-EOT
host:
  bob:
    ensure: present
      EOT
      )
      expect do
        process_a_node('node_name', 'apply', resource_file, false)
      end.to raise_error(Exception, 'Somehow got a report with a different number of faillures')
    end
  end

end
