require 'spec_helper'
require 'puppet/indirector/node/yaml_device.rb'
require 'tempfile'

describe 'yaml device node indirection' do

  let :node_indirection do
    Puppet::Node::YamlDevice.new
  end

  def get_request(key)
    Puppet::Indirector::Request.new(
      'node', 'find', key, nil
    )
  end

  def create_device_file(contents)
    tmp_file = Tempfile.new('tmp_file')
    path = tmp_file.path
    node_indirection.class.expects(:device_file).with('node_name').returns(path)
    File.open(path, 'w') do |fh|
      fh.write(contents)
    end
  end

  it 'should fail when there is not device.yaml file' do
    tmp_file = Tempfile.new('tmp_file')
    path = tmp_file.path
    tmp_file.unlink
    Puppet.expects(:warning).with('Yaml file /dev/null/node_data/node_name.yaml does not exist')
    node_indirection.find(get_request('node_name'))
  end

  it 'should fail when yaml file is empty' do
    tmp_file = Tempfile.new('tmp_file')
    node_indirection.class.expects(:device_file).returns(tmp_file.path)
    expect do
      node_indirection.find(get_request('node_name'))
    end.to raise_error(Puppet::Error, /was empty/)
  end

  it 'should set parameters in node object' do
    create_device_file(<<-EOT
node_name:
  parameters:
    foo: bar
    EOT
    )
    node_indirection.find(get_request('node_name')).parameters['foo'].should == 'bar'
  end

  it 'should accpet classes as array or hash' do
    create_device_file(<<-EOT
node_name:
  classes:
    foo:
      one: two
    bar:
      one: two
    EOT
    )
    node_indirection.find(get_request('node_name')).classes['foo'].should == {'one' => 'two'}
  end

  it 'should accept resources and set them in asm::resource_wrapper' do
    create_device_file(<<-EOT
node_name:
  resources:
    user:
      dan:
        ensure: present
    EOT
    )
    node_indirection.find(get_request('node_name')).classes['asm::resource_wrapper']['resources'].should == {
      'user' => {'dan' => {'ensure' => 'present'}}
    }
  end

  it 'should merge resources setting class when a hash of classes was passed' do
    create_device_file(<<-EOT
node_name:
  classes:
    foo:
      one: two
    bar:
      one: two
  resources:
    user:
      dan:
        ensure: present
    EOT
    )
    node = node_indirection.find(get_request('node_name'))
    node.classes['foo'].should == {'one' => 'two'}
    node.classes['asm::resource_wrapper']['resources'].should == {
      'user' => {'dan' => {'ensure' => 'present'}}
    }
  end

  it 'should fail it resources and an array of classes was passed' do
    create_device_file(<<-EOT
node_name:
  classes:
    - foo
    - bar
  resources:
    user:
      dan:
        ensure: present
    EOT
    )
    expect do
      node_indirection.find(get_request('node_name'))
    end.to raise_error(Puppet::Error, /Cannot add resources if an array of classes was passed/)
  end

end
